# Two guys, one swap.
#
# Rough statement of the so called "Two envelopes paradox" :
#
# Some guy provides a pair of envelopes containing money, one of which contains
# twice the amount of the other one. The player is given, at random, one of the
# envelopes. He may or may not be allowed to check the amount of money it
# contains. He then has to choose wether or not he wants to swap his current
# enveloppe for the other one, in hope of a bigger gain.
#
# The question being : is there reason for him to swap, and if so, which one ?
#
# This statement is confusing as hell, and I fell right for it.
# Thanks to Draklaw and DS for help in clearing that up.
#
# The heart of the ambiguity lies on the the fact that the second envelope,
# containing either twice or half the amount of the first one, may or may not
# exist at the time the player makes its choice.

import random

# This player thinks the grass is greener on the other side.
def switcher(amount):
	return True

# This player wants things to stay the way they are.
def keeper(amount):
	return False

# This guy merely provides the contents of the second envelope.
# Which is either half or twice what the player has.
def actual_random_guy(wanna_switch):
	# In any real-world scenario, there are only two amounts to consider.
	envelopes = random.choice([[5,10],[10,20]])

	handed = random.choice([0,1])

	if wanna_switch(envelopes[handed]):
		return envelopes[1-handed]
	else:
		return envelopes[handed]
	

# This guy pulls a second envelope from his vest.
# Which contains either half or twice what the player has.
def magical_random_guy(wanna_switch):
	# This guy has yet only provided one envelope when the player makes his call.
	envelope = 10
	
	if wanna_switch(envelope):
		# He produces a second envelope, freely doubling or halving the gain.
		mysterious_second_envelope = random.choice([envelope//2,envelope*2])
		return mysterious_second_envelope
	else:
		return envelope
		# Magical guy always produces a second envelope, as per spec - however...
		mysterious_second_envelope = random.choice([envelope//2,envelope*2])
		# The player is gone. If a tree falls in a forest and no-one is around...
		# ...will the compiler optimize out the falling noise ? Guess so.

result = {
	"actual guy"  : actual_random_guy,
	"magical guy" : magical_random_guy
}

def play_a_lot(opponent):
	switcher_gains = 0
	keeper_gains = 0
	for i in range(1000):
		switcher_gains += result[opponent](switcher)
	for j in range(1000):
		keeper_gains += result[opponent](keeper)
	print ("Playing against the",opponent,"...")
	print ()
	print ("The switchers collectively won",switcher_gains,"$.");
	print ("The keepers collectively won",keeper_gains,"$.");

print ()
play_a_lot("actual guy")
print ()
print ()
play_a_lot("magical guy")

# TRWTF is, of course, players not giving a fuck about how much they got.
